package com.unissoft.intrusion;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.unissoft.common.DateUtils;
import com.unissoft.common.MyConstant;
import com.unissoft.utils.RedisUtil;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.tomcat.util.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author qingsong.han
 * @description:
 * @create 2022-11-07 10:21
 */
public interface IntrusionService {

    void pushIntrusion(Integer sAlarmType, String alarmNow, String deviceIP, String orgCode,
                       String cameraIP, String picByte, Integer size);

    void editChannelCode(Map<String, String> channelCodes);


    @Data
    @Component
    class IntrusionModel {
        // 报警类型
        private Integer alarmType;
        // 报警时间
        private String alarmNow;
        // 超脑设备ip
        private String deviceIP;
        // 机构编码
        private String orgCode;
        // ICC通道号
        private String channelCode;
        //图片base64格式字节
        private String picByte;
        //图片大小
        private Integer picSize;
        //报警摄像头IP
        private String cameraIP;
    }

    // 定义内部类
    @Slf4j
    @Service
    class IntrusionServiceImpl implements IntrusionService {
        @Resource
        private RedisUtil redisUtil;
        /*@Resource
        private RestTemplate restTemplate;*/
        // 推送地址
        /*@Value("${address.pushToIntrusion}")
        private String pushToIntrusion;*/
        @Value("${spring.redis.ten-minutes}")
        private long tenMinutes;
        @Resource
        private IntrusionModel intrusionModel;

        @Override
        public void pushIntrusion(Integer sAlarmType, String alarmNow, String deviceIP, String orgCode,
                                  String cameraIP, String picByte, Integer size) {

            // TODO 物联网到业务平台推送数据网络不通,修改为业务平台主动获取数据
            // 上报数据-接口
            //
            log.info("接收到报警数据，类型：" + sAlarmType + "\t时间：" + alarmNow + "\t超脑IP：" + deviceIP);
            SimpleDateFormat sf = new SimpleDateFormat("yyyyMMddHHmmss");
            String newName = sf.format(new Date());
            String filename = "";
            FileOutputStream fout;
            byte[] bytes = Base64.decodeBase64(picByte);
            log.info("pic size:{}", bytes.length);
            try {
                File file = new File( "." + File.separator + "mypic" + File.separator);
                if(!file.exists())
                    file.mkdir();
                filename = "." + File.separator + "mypic" + File.separator + newName + "VCA_TRAVERSE_PLANE" + ".jpg";
                file = new File(filename);
                file.createNewFile();
                fout = new FileOutputStream(file);
                //将字节写入文件
                fout.write(bytes);
                fout.close();
                System.out.println("pic save ok");
            } catch (FileNotFoundException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }finally {
                try {
                    if (fout != null)
                        fout.close();
                }catch (IOException ioe){
                    ioe.printStackTrace();
                }
            }



            intrusionModel.setDeviceIP(deviceIP);
            intrusionModel.setAlarmType(sAlarmType);
            intrusionModel.setAlarmNow(alarmNow);
            intrusionModel.setOrgCode(orgCode);
            //intrusionModel.setPicByte(picByte);
            intrusionModel.setCameraIP(cameraIP);
            // 添加通道号
            intrusionModel.setChannelCode(this.getChannelCode(deviceIP));
            // 设置异常闯入设备数据-hashCode
            redisUtil.hset(MyConstant.INTRUSION_ALARM_IP, deviceIP, intrusionModel, tenMinutes);
            log.info("Up it!-> sAlarmType: {} alarmNow: {} deviceIP: {} orgCode: {}", sAlarmType, alarmNow, deviceIP, orgCode);
        }

        private String getChannelCode(String deviceIP) {

            Object oIP = redisUtil.hget(MyConstant.ICC_CHANNEL_CODE, deviceIP);

            if (oIP != null) {

                String channelCode = redisUtil.hget(MyConstant.ICC_CHANNEL_CODE, deviceIP).toString();
                if (StringUtils.isEmpty(channelCode))
                    channelCode = this.staticChannelCode(deviceIP);
                return channelCode;
            }else {
                log.info("VALUE OF MyConstant.ICC_CHANNEL_CODE IS NULL");
                return null;
            }
        }

        private String staticChannelCode(String deviceIP) {
            log.info("IntrusionServiceImpl.staticChannelCode");
            switch (deviceIP) {
                case "192.168.193.1":
                    return "1000384$1$0$1";
                case "192.168.193.16":
                    return "1000384$1$0$10";
                case "192.168.193.17":
                    return "1000384$1$0$8";
                case "192.168.143.10":
                    return "1000365$1$0$0";
                case "192.168.143.11":
                    return "1000365$1$0$1";
                case "192.168.163.21":
                    return "1000445$1$0$25";
                case "192.168.153.15":
                    return "1000462$1$0$6";
                case "192.168.153.16":
                    return "1000462$1$0$5";
                case "192.168.153.20":
                    return "1000462$1$0$10";
                case "192.168.153.22":
                    return "1000462$1$0$12";
                case "192.168.153.23":
                    return "1000462$1$0$13";
                case "192.168.153.25":
                    return "1000462$1$0$15";
                case "192.168.173.162":
                    return "1000221$1$0$0";
                case "192.168.173.163":
                    return "1000220$1$0$0";
                case "192.168.173.164":
                    return "1000200$1$0$0";
                default:
                    return "";
            }
        }

        @Override
        public void editChannelCode(Map<String, String> channelCodes) {
            channelCodes.forEach((ip, code) ->
                    redisUtil.hset(MyConstant.ICC_CHANNEL_CODE, ip, code)
            );
        }

    }


}
